# coding: utf-8

# flake8: noqa
"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.24.0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

# import models into model package
from dragonk8s.dragon.models.io_dragon_apps_v1_job import IoDragonAppsV1Job
from dragonk8s.dragon.models.io_dragon_apps_v1_job_list import IoDragonAppsV1JobList
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec import IoDragonAppsV1JobSpec
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_selector import IoDragonAppsV1JobSpecSelector
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_selector_match_expressions import IoDragonAppsV1JobSpecSelectorMatchExpressions
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template import IoDragonAppsV1JobSpecTemplate
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec import IoDragonAppsV1JobSpecTemplateSpec
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_affinity import IoDragonAppsV1JobSpecTemplateSpecAffinity
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_affinity_node_affinity import IoDragonAppsV1JobSpecTemplateSpecAffinityNodeAffinity
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_affinity_node_affinity_preference import IoDragonAppsV1JobSpecTemplateSpecAffinityNodeAffinityPreference
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_affinity_node_affinity_preference_match_expressions import IoDragonAppsV1JobSpecTemplateSpecAffinityNodeAffinityPreferenceMatchExpressions
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_affinity_node_affinity_preferred_during_scheduling_ignored_during_execution import IoDragonAppsV1JobSpecTemplateSpecAffinityNodeAffinityPreferredDuringSchedulingIgnoredDuringExecution
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_affinity_node_affinity_required_during_scheduling_ignored_during_execution import IoDragonAppsV1JobSpecTemplateSpecAffinityNodeAffinityRequiredDuringSchedulingIgnoredDuringExecution
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_affinity_node_affinity_required_during_scheduling_ignored_during_execution_node_selector_terms import IoDragonAppsV1JobSpecTemplateSpecAffinityNodeAffinityRequiredDuringSchedulingIgnoredDuringExecutionNodeSelectorTerms
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_affinity_pod_affinity import IoDragonAppsV1JobSpecTemplateSpecAffinityPodAffinity
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_affinity_pod_affinity_pod_affinity_term import IoDragonAppsV1JobSpecTemplateSpecAffinityPodAffinityPodAffinityTerm
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_affinity_pod_affinity_pod_affinity_term_label_selector import IoDragonAppsV1JobSpecTemplateSpecAffinityPodAffinityPodAffinityTermLabelSelector
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_affinity_pod_affinity_pod_affinity_term_namespace_selector import IoDragonAppsV1JobSpecTemplateSpecAffinityPodAffinityPodAffinityTermNamespaceSelector
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_affinity_pod_affinity_preferred_during_scheduling_ignored_during_execution import IoDragonAppsV1JobSpecTemplateSpecAffinityPodAffinityPreferredDuringSchedulingIgnoredDuringExecution
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_affinity_pod_affinity_required_during_scheduling_ignored_during_execution import IoDragonAppsV1JobSpecTemplateSpecAffinityPodAffinityRequiredDuringSchedulingIgnoredDuringExecution
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_affinity_pod_anti_affinity import IoDragonAppsV1JobSpecTemplateSpecAffinityPodAntiAffinity
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_aws_elastic_block_store import IoDragonAppsV1JobSpecTemplateSpecAwsElasticBlockStore
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_azure_disk import IoDragonAppsV1JobSpecTemplateSpecAzureDisk
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_azure_file import IoDragonAppsV1JobSpecTemplateSpecAzureFile
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_cephfs import IoDragonAppsV1JobSpecTemplateSpecCephfs
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_cephfs_secret_ref import IoDragonAppsV1JobSpecTemplateSpecCephfsSecretRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_cinder import IoDragonAppsV1JobSpecTemplateSpecCinder
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_cinder_secret_ref import IoDragonAppsV1JobSpecTemplateSpecCinderSecretRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_config_map import IoDragonAppsV1JobSpecTemplateSpecConfigMap
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_config_map_items import IoDragonAppsV1JobSpecTemplateSpecConfigMapItems
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_config_map_ref import IoDragonAppsV1JobSpecTemplateSpecConfigMapRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_containers import IoDragonAppsV1JobSpecTemplateSpecContainers
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_csi import IoDragonAppsV1JobSpecTemplateSpecCsi
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_csi_node_publish_secret_ref import IoDragonAppsV1JobSpecTemplateSpecCsiNodePublishSecretRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_dns_config import IoDragonAppsV1JobSpecTemplateSpecDnsConfig
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_dns_config_options import IoDragonAppsV1JobSpecTemplateSpecDnsConfigOptions
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_downward_api import IoDragonAppsV1JobSpecTemplateSpecDownwardAPI
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_downward_api_field_ref import IoDragonAppsV1JobSpecTemplateSpecDownwardAPIFieldRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_downward_api_items import IoDragonAppsV1JobSpecTemplateSpecDownwardAPIItems
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_downward_api_resource_field_ref import IoDragonAppsV1JobSpecTemplateSpecDownwardAPIResourceFieldRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_empty_dir import IoDragonAppsV1JobSpecTemplateSpecEmptyDir
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_env import IoDragonAppsV1JobSpecTemplateSpecEnv
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_env_from import IoDragonAppsV1JobSpecTemplateSpecEnvFrom
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_ephemeral import IoDragonAppsV1JobSpecTemplateSpecEphemeral
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_ephemeral_containers import IoDragonAppsV1JobSpecTemplateSpecEphemeralContainers
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_ephemeral_volume_claim_template import IoDragonAppsV1JobSpecTemplateSpecEphemeralVolumeClaimTemplate
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_ephemeral_volume_claim_template_spec import IoDragonAppsV1JobSpecTemplateSpecEphemeralVolumeClaimTemplateSpec
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_ephemeral_volume_claim_template_spec_data_source import IoDragonAppsV1JobSpecTemplateSpecEphemeralVolumeClaimTemplateSpecDataSource
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_ephemeral_volume_claim_template_spec_data_source_ref import IoDragonAppsV1JobSpecTemplateSpecEphemeralVolumeClaimTemplateSpecDataSourceRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_ephemeral_volume_claim_template_spec_resources import IoDragonAppsV1JobSpecTemplateSpecEphemeralVolumeClaimTemplateSpecResources
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_ephemeral_volume_claim_template_spec_selector import IoDragonAppsV1JobSpecTemplateSpecEphemeralVolumeClaimTemplateSpecSelector
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_fc import IoDragonAppsV1JobSpecTemplateSpecFc
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_flex_volume import IoDragonAppsV1JobSpecTemplateSpecFlexVolume
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_flex_volume_secret_ref import IoDragonAppsV1JobSpecTemplateSpecFlexVolumeSecretRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_flocker import IoDragonAppsV1JobSpecTemplateSpecFlocker
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_gce_persistent_disk import IoDragonAppsV1JobSpecTemplateSpecGcePersistentDisk
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_git_repo import IoDragonAppsV1JobSpecTemplateSpecGitRepo
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_glusterfs import IoDragonAppsV1JobSpecTemplateSpecGlusterfs
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_host_aliases import IoDragonAppsV1JobSpecTemplateSpecHostAliases
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_host_path import IoDragonAppsV1JobSpecTemplateSpecHostPath
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_image_pull_secrets import IoDragonAppsV1JobSpecTemplateSpecImagePullSecrets
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_iscsi import IoDragonAppsV1JobSpecTemplateSpecIscsi
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_iscsi_secret_ref import IoDragonAppsV1JobSpecTemplateSpecIscsiSecretRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_label_selector import IoDragonAppsV1JobSpecTemplateSpecLabelSelector
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_lifecycle import IoDragonAppsV1JobSpecTemplateSpecLifecycle
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_lifecycle1 import IoDragonAppsV1JobSpecTemplateSpecLifecycle1
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_lifecycle_post_start import IoDragonAppsV1JobSpecTemplateSpecLifecyclePostStart
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_lifecycle_post_start_exec import IoDragonAppsV1JobSpecTemplateSpecLifecyclePostStartExec
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_lifecycle_post_start_http_get import IoDragonAppsV1JobSpecTemplateSpecLifecyclePostStartHttpGet
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_lifecycle_post_start_http_get_http_headers import IoDragonAppsV1JobSpecTemplateSpecLifecyclePostStartHttpGetHttpHeaders
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_lifecycle_post_start_tcp_socket import IoDragonAppsV1JobSpecTemplateSpecLifecyclePostStartTcpSocket
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_lifecycle_pre_stop import IoDragonAppsV1JobSpecTemplateSpecLifecyclePreStop
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_liveness_probe import IoDragonAppsV1JobSpecTemplateSpecLivenessProbe
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_liveness_probe1 import IoDragonAppsV1JobSpecTemplateSpecLivenessProbe1
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_liveness_probe_grpc import IoDragonAppsV1JobSpecTemplateSpecLivenessProbeGrpc
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_liveness_probe_tcp_socket import IoDragonAppsV1JobSpecTemplateSpecLivenessProbeTcpSocket
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_nfs import IoDragonAppsV1JobSpecTemplateSpecNfs
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_os import IoDragonAppsV1JobSpecTemplateSpecOs
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_persistent_volume_claim import IoDragonAppsV1JobSpecTemplateSpecPersistentVolumeClaim
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_photon_persistent_disk import IoDragonAppsV1JobSpecTemplateSpecPhotonPersistentDisk
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_ports import IoDragonAppsV1JobSpecTemplateSpecPorts
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_portworx_volume import IoDragonAppsV1JobSpecTemplateSpecPortworxVolume
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_projected import IoDragonAppsV1JobSpecTemplateSpecProjected
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_projected_config_map import IoDragonAppsV1JobSpecTemplateSpecProjectedConfigMap
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_projected_downward_api import IoDragonAppsV1JobSpecTemplateSpecProjectedDownwardAPI
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_projected_secret import IoDragonAppsV1JobSpecTemplateSpecProjectedSecret
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_projected_service_account_token import IoDragonAppsV1JobSpecTemplateSpecProjectedServiceAccountToken
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_projected_sources import IoDragonAppsV1JobSpecTemplateSpecProjectedSources
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_quobyte import IoDragonAppsV1JobSpecTemplateSpecQuobyte
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_rbd import IoDragonAppsV1JobSpecTemplateSpecRbd
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_rbd_secret_ref import IoDragonAppsV1JobSpecTemplateSpecRbdSecretRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_readiness_gates import IoDragonAppsV1JobSpecTemplateSpecReadinessGates
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_readiness_probe import IoDragonAppsV1JobSpecTemplateSpecReadinessProbe
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_resources import IoDragonAppsV1JobSpecTemplateSpecResources
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_resources1 import IoDragonAppsV1JobSpecTemplateSpecResources1
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_scale_io import IoDragonAppsV1JobSpecTemplateSpecScaleIO
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_scale_io_secret_ref import IoDragonAppsV1JobSpecTemplateSpecScaleIOSecretRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_secret import IoDragonAppsV1JobSpecTemplateSpecSecret
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_secret_ref import IoDragonAppsV1JobSpecTemplateSpecSecretRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_security_context import IoDragonAppsV1JobSpecTemplateSpecSecurityContext
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_security_context1 import IoDragonAppsV1JobSpecTemplateSpecSecurityContext1
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_security_context2 import IoDragonAppsV1JobSpecTemplateSpecSecurityContext2
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_security_context2_se_linux_options import IoDragonAppsV1JobSpecTemplateSpecSecurityContext2SeLinuxOptions
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_security_context2_seccomp_profile import IoDragonAppsV1JobSpecTemplateSpecSecurityContext2SeccompProfile
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_security_context2_sysctls import IoDragonAppsV1JobSpecTemplateSpecSecurityContext2Sysctls
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_security_context2_windows_options import IoDragonAppsV1JobSpecTemplateSpecSecurityContext2WindowsOptions
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_security_context_capabilities import IoDragonAppsV1JobSpecTemplateSpecSecurityContextCapabilities
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_security_context_se_linux_options import IoDragonAppsV1JobSpecTemplateSpecSecurityContextSeLinuxOptions
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_security_context_seccomp_profile import IoDragonAppsV1JobSpecTemplateSpecSecurityContextSeccompProfile
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_security_context_windows_options import IoDragonAppsV1JobSpecTemplateSpecSecurityContextWindowsOptions
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_startup_probe import IoDragonAppsV1JobSpecTemplateSpecStartupProbe
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_storageos import IoDragonAppsV1JobSpecTemplateSpecStorageos
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_storageos_secret_ref import IoDragonAppsV1JobSpecTemplateSpecStorageosSecretRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_tolerations import IoDragonAppsV1JobSpecTemplateSpecTolerations
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_topology_spread_constraints import IoDragonAppsV1JobSpecTemplateSpecTopologySpreadConstraints
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_value_from import IoDragonAppsV1JobSpecTemplateSpecValueFrom
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_value_from_config_map_key_ref import IoDragonAppsV1JobSpecTemplateSpecValueFromConfigMapKeyRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_value_from_field_ref import IoDragonAppsV1JobSpecTemplateSpecValueFromFieldRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_value_from_resource_field_ref import IoDragonAppsV1JobSpecTemplateSpecValueFromResourceFieldRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_value_from_secret_key_ref import IoDragonAppsV1JobSpecTemplateSpecValueFromSecretKeyRef
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_volume_devices import IoDragonAppsV1JobSpecTemplateSpecVolumeDevices
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_volume_mounts import IoDragonAppsV1JobSpecTemplateSpecVolumeMounts
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_volumes import IoDragonAppsV1JobSpecTemplateSpecVolumes
from dragonk8s.dragon.models.io_dragon_apps_v1_job_spec_template_spec_vsphere_volume import IoDragonAppsV1JobSpecTemplateSpecVsphereVolume
from dragonk8s.dragon.models.io_dragon_apps_v1_job_status import IoDragonAppsV1JobStatus
from dragonk8s.dragon.models.io_dragon_apps_v1_job_status_conditions import IoDragonAppsV1JobStatusConditions
from dragonk8s.dragon.models.io_dragon_apps_v1_job_status_uncounted_terminated_pods import IoDragonAppsV1JobStatusUncountedTerminatedPods
from dragonk8s.dragon.models.io_dragon_apps_v1_replica_set import IoDragonAppsV1ReplicaSet
from dragonk8s.dragon.models.io_dragon_apps_v1_replica_set_list import IoDragonAppsV1ReplicaSetList
from dragonk8s.dragon.models.io_dragon_apps_v1_replica_set_spec import IoDragonAppsV1ReplicaSetSpec
from dragonk8s.dragon.models.io_dragon_apps_v1_replica_set_status import IoDragonAppsV1ReplicaSetStatus
from dragonk8s.dragon.models.io_dragon_apps_v1_replica_set_status_conditions import IoDragonAppsV1ReplicaSetStatusConditions
